
import sys
import datetime
from collections import namedtuple


ALWAYS = -1
NOTSET = 0
DEBUG = 10
INFO = 20
WARNING = 30
ERROR = 40
CRITICAL = 50
CATASTROPHIC = 1000

NO_COLOR = 'none'


def _color(code):
    """A simple ANSI color wrapper factory"""
    return lambda t: '\033[{0}{1}\033[0;m'.format(code, t)


red = _color('1;31m')
green = _color('1;32m')
yellow = _color('1;33m')
blue = _color('1;34m')
purple = _color('1;35m')
cyan = _color('1;36m')
white = _color('1;37m')

_str_to_color = {
    'red': red,
    'green': green,
    'yellow': yellow,
    'blue': blue,
    'purple': purple,
    'cyan': cyan,
    'white': white,
    NO_COLOR: lambda x: x
}

base_color_config = {
    'debug': 'white',
    'info': 'blue',
    'warning': 'yellow',
    'error': 'red',
    'critical': 'red',
    'catastrophic': 'red',
}


def colorize(message, color, default=NO_COLOR):
    # TODO - not sure if I like this... KeyError might be better. If change to a KeyError relevant
    #  test is tests.test_plog.TestColorize.test_colorize_default().
    if color not in _str_to_color:
        color = default
    color_func = _str_to_color[color]
    return color_func(message)


Log = namedtuple('Log', ['when', 'level', 'message'])


class PrettyLogger:
    def __init__(self, level=NOTSET):
        self.level = level
        self.use_colors = False
        self.color_config = base_color_config
        self.logs = []

    def config_style(self, use_colors=False, color_config=base_color_config):
        self.use_colors = use_colors
        self.color_config = color_config

    def _should_output(self, level=NOTSET):
        return level >= self.level or level == ALWAYS

    def log(self, message, level=NOTSET, color=NO_COLOR, write_to='stdout'):
        if type(write_to) == str:
            write_to = {'stdout': sys.stdout, 'stderr': sys.stderr}[write_to]
        self.logs.append(Log(datetime.datetime.now(), level, message))
        if self.use_colors:
            message = colorize(message, color)
        if self._should_output(level):
            write_to.write(f'{message}\n')

    def debug(self, message='debug'):
        color = self.color_config['debug']
        self.log(message, level=DEBUG, color=color)

    def info(self, message):
        color = self.color_config['info']
        self.log(message, level=INFO, color=color)

    def warning(self, message='warning'):
        color = self.color_config['warning']
        self.log(message, level=WARNING, color=color, write_to='stderr')

    def error(self, message='error'):
        color = self.color_config['error']
        self.log(message, level=ERROR, color=color, write_to='stderr')

    def critical(self, message):
        color = self.color_config['critical']
        self.log(message, level=CRITICAL, color=color, write_to='stderr')

    def catastrophic(self, message):
        color = self.color_config['catastrophic']
        self.log(message, level=CATASTROPHIC, color=color, write_to='stderr')


class QuietLogger(PrettyLogger):
    def __init__(self):
        super().__init__()

    def _should_output(self, level=NOTSET):
        return False
