import os
import sys

from invoke import task as task_
from invoke.collection import Collection

from .plog import base_color_config
from .plog import PrettyLogger


class TaskManager:

    def __init__(self, project_root='', source_root='src', logger=PrettyLogger(),
                 tasks_module='tasks'):
        self.project_root = project_root
        self.source_root = source_root
        self.logger = logger
        self.tasks_module = tasks_module
        self._collection = Collection()
        self._load_module_tasks(tasks_module)

    def __getitem__(self, item):
        return self._collection.__getitem__(item)

    def add_task(self, task_name='', task=None, lib=None, cmd='', doc=None, aliases=list(),
                 env=None, **kwargs):
        if task:
            t = task
        elif cmd != '':
            t = _basic_command_task_factory(task_name, cmd, **kwargs)
        else:
            t = self._get_task_from_module(task_name, lib=lib, **kwargs)

        if doc is not None:
            t.__doc__ = doc

        if t is not None:
            t.aliases += tuple(aliases)
            self._collection.add_task(t, name=task_name)

        return t

    def namespace(self, tasks_module=None):
        self._load_module_tasks(self.tasks_module if tasks_module is None else tasks_module)
        return self._collection

    @staticmethod
    def from_spec(spec):
        proj_root = spec.pop('proj_root', '')
        source_root = spec.pop('source_root', 'src')
        logger = spec.pop('logger', PrettyLogger())
        tasks_module = spec.pop('tasks_module', 'tasks')
        mgr = TaskManager(proj_root, source_root, logger, tasks_module)

        if 'style' in spec:
            style = spec.pop('style')
            use_colors = style.pop('use_colors', False)
            color_config = style.pop('color_config', base_color_config)
            logger.config_style(use_colors, color_config)

        if 'tasks' in spec:
            tasks = spec.pop('tasks')
            for task_name, t in tasks.items():
                if 'dir' in t:
                    t['dir_'] = t.pop('dir')
                mgr.add_task(task_name=task_name, **t)

        return mgr

    def _get_task_from_module(self, task_name='', lib=None, **kwargs):
        try:
            lib = task_name if lib is None else lib
            mod = __import__(f'prettyrepo.lib.{lib}_', fromlist=[''])
            if hasattr(mod, 'get_task'):
                return mod.get_task(self, task_name=task_name, **kwargs)
            else:  # pragma: no cover
                raise AssertionError('internal bug - missing get_task')
        except ModuleNotFoundError:
            self.logger.error(f'no module found for {lib} task!')
        except TypeError as e:  # pragma: no cover
            raise AssertionError(f'incorrect parameters supplied to lib get_task - {str(e)}')
        return None

    def _load_module_tasks(self, tasks_module):
        if type(tasks_module) == str:
            tasks_module = sys.modules[tasks_module]
        if tasks_module is not None:
            c = Collection.from_module(tasks_module)
            for t in c.tasks:
                self._collection.add_task(c[t])


def _basic_command_task_factory(task_name, cmd, dir_=None, run_from=None):

    if run_from is None:
        run_from = dir_

    @task_(name=task_name)
    def t(ctx, no_echo=True):
        with ctx.cd(run_from if run_from else os.getcwd()):
            return ctx.run(cmd, echo=not no_echo)

    return t
